@@ -985,6 +985,12 @@ static unicap_status_t v4l2_set_format( void *cpi_data, unicap_format_t *_format
 
    unicap_format_t format;
 
+   int old_running = handle->capture_running;
+   
+   if( handle->capture_running ){
+      v4l2_capture_stop( handle );
+   }
+
    unicap_copy_format( &format, _format );
 
    TRACE( "v4l2_set_format\n" );
@@ -1032,6 +1038,10 @@ static unicap_status_t v4l2_set_format( void *cpi_data, unicap_format_t *_format
       return STATUS_FAILURE;
    }
 
+   if( old_running ){
+      v4l2_capture_start( handle );
+   }
+   
 
 
    return STATUS_SUCCESS;
