@@ -36,6 +36,8 @@
 #include <unicap_status.h>
 #include <unicap_version.h>
 
+#define UNICAP_MAX_DEVICES 128
+
 
 /**
  * unicap_buffer_type_t:
@@ -58,8 +60,8 @@ typedef enum
 } unicap_buffer_flags_t;
 
 /**
- * unicap_device_t:
- * @identifier: unique textual if of a device
+ * unicap_device_t: 
+ * @identifier: unique textual ID of a device
  * @model_name: model name of the device
  * @vendor_name: name of the device vendor
  * @model_id: unique model id, ie. serial number
@@ -67,6 +69,8 @@ typedef enum
  * @cpi_layer: name of the plugin used to communicate with the device
  * @device: name of the device file, if any
  * @flags: 
+ * 
+ * a struct defining a device
  */
 struct unicap_device_t
 {
@@ -83,7 +87,6 @@ struct unicap_device_t
 	unsigned int flags;
 };
 
-
 typedef struct unicap_device_t unicap_device_t;
 
 /**
@@ -769,7 +772,7 @@ unicap_status_t unicap_queue_buffer( unicap_handle_t handle, unicap_data_buffer_
 unicap_status_t unicap_dequeue_buffer( unicap_handle_t handle, unicap_data_buffer_t **data_buffer );
 
 /**
- * unicap_wait_buffer
+ * unicap_wait_buffer:
  * @handle:
  * @data_buffer:
  * 
@@ -802,16 +805,40 @@ unicap_status_t unicap_void_property( unicap_property_t *property );
 unicap_format_t *unicap_copy_format( unicap_format_t *dest, const unicap_format_t *src );
 unicap_property_t *unicap_copy_property( unicap_property_t *dest, const unicap_property_t *src );
 
-/*
-  Makes calling process the stream owner for this device
+
+#ifdef UNICAP_FLAGS_NOT_AS_ENUM 
+#define UNICAP_FLAGS_LOCKED               ( 1<<0 )
+#define UNICAP_FLAGS_LOCK_CURRENT_PROCESS ( 1<<1 )
+#else
+typedef enum
+{
+   UNICAP_FLAGS_LOCKED =                  ( 1<< 0 ),
+   UNICAP_FLAGS_LOCK_CURRENT_PROCESS =    ( 1<< 1 ),
+} unicap_lock_flags_t;
+#endif
+   
+/**
+ * unicap_lock_stream:
+ * @handle:
+ *
  */
 unicap_status_t unicap_lock_stream( unicap_handle_t handle );
+
+/**
+ * unicap_lock_stream:
+ * @handle:
+ *
+ */
 unicap_status_t unicap_unlock_stream( unicap_handle_t handle );
+
+/**
+ * unicap_is_stream_locked: 
+ * @device:
+ *
+ * 
+ */
 int unicap_is_stream_locked( unicap_device_t *device );
 	
-/*
-  Makes calling process the properties owner
- */
 unicap_status_t unicap_lock_properties( unicap_handle_t handle );
 	
 unicap_status_t unicap_unlock_properties( unicap_handle_t handle );
@@ -823,6 +850,10 @@ void unicap_cpi_register_event_notification( void *,
  */
 void unicap_cache_init( void );
 
+int unicap_get_ref_count( unicap_handle_t handle );
+
+
+
 
 
 UNICAP_END_DECLS
