@@ -66,16 +66,43 @@ union semun_linux {
 				  (Linux specific) */
 };
 
+struct unicap_device_cache{
+   unicap_device_t device;
+   unicap_handle_t handle;
+};
+
+   
+
+
+
 static unicap_handle_t unicap_copy_handle( unicap_handle_t handle );
+static struct unicap_device_cache *lookup_device_cache( unicap_device_t *device );
+
 
 const unsigned int unicap_major_version=UNICAP_MAJOR_VERSION;
 const unsigned int unicap_minor_version=UNICAP_MINOR_VERSION;
 const unsigned int unicap_micro_version=UNICAP_MICRO_VERSION;
 
 static int g_device_count = 0;
-static unicap_device_t g_devices[128];
+static unicap_device_t g_devices[UNICAP_MAX_DEVICES];
 static int g_filter_remote = 0;
 
+static struct unicap_device_cache g_device_cache[UNICAP_MAX_DEVICES];
+static volatile int g_initialized = 0;
+
+
+static void unicap_initialize( void )
+{
+   int i;
+   if( !g_initialized ){
+      for( i = 0; i < UNICAP_MAX_DEVICES; i++ ){
+	 unicap_void_device( &g_device_cache[i].device );
+	 g_device_cache[i].handle = NULL;
+      }
+      
+      g_initialized = 1;
+   }
+}
 
 #if ENABLE_STATIC_CPI
 static int g_static_cpi_initialized = 0;
@@ -104,6 +131,44 @@ static void initialize_static_cpis()
 }
 #endif
 
+static struct unicap_device_cache *lookup_device_cache( unicap_device_t *device )
+{
+   int i;
+   for( i = 0; i < UNICAP_MAX_DEVICES; i++ ){
+      if( _check_device_match( device, &g_device_cache[i].device ) ){
+	 return &g_device_cache[i];
+      }
+   }
+   
+   return NULL;
+}
+
+static void store_device_cache( unicap_device_t *device, unicap_handle_t handle )
+{
+   int i;
+   
+   for( i = 0; i < UNICAP_MAX_DEVICES; i++ ){
+      if( g_device_cache[i].handle == NULL ){
+	 memcpy( &g_device_cache[i].device, device, sizeof( unicap_device_t ) );
+	 g_device_cache[i].handle = unicap_clone_handle( handle );
+	 break;
+      }
+   }
+}
+
+static void free_device_cache_entries( void )
+{
+   int i;
+
+   for( i = 0; i < UNICAP_MAX_DEVICES; i++ ){
+      if( g_device_cache[i].handle && ( *g_device_cache[i].handle->ref_count == 1 ) ){
+	 unicap_close( g_device_cache[i].handle );
+	 unicap_void_device( &g_device_cache[i].device );
+	 g_device_cache[i].handle = NULL;
+      }
+   }   
+}
+
 unicap_status_t unicap_set_filter_remote( int do_filter )
 {
    if( do_filter )
@@ -162,7 +227,7 @@ static unicap_status_t _unicap_release_cpi_lock( int sem_id )
 static void unicap_event_callback( unicap_handle_t handle, unicap_event_t event, ... )
 {
 
-   if( !handle->cb_info[event].func )
+   if( !handle->cb_info || !handle->cb_info[event].func )
    {
       return;
    }
@@ -226,6 +291,8 @@ unicap_status_t unicap_reenumerate_devices( int *_pcount )
 {
    int count = 128;
 
+   unicap_initialize();
+
    unicap_real_enumerate_devices( &count );
 	
    g_device_count = count;
@@ -242,6 +309,8 @@ unicap_status_t unicap_enumerate_devices( unicap_device_t *specifier, unicap_dev
 {
    int current_index = -1;
 	
+   unicap_initialize();
+
    if( g_device_count != 0)
    {
       int i;
@@ -266,7 +335,7 @@ unicap_status_t unicap_enumerate_devices( unicap_device_t *specifier, unicap_dev
    } 
    else 
    {
-      if( SUCCESS( unicap_reenumerate_devices( &g_device_count ) ) )
+      if( SUCCESS( unicap_reenumerate_devices( NULL ) ) )
       {
 	 if( g_device_count )
 	 {
@@ -293,7 +362,7 @@ static unicap_status_t enumerate_devices( int *count )
    struct dirent *cpi_dirent;
    char *dirname;
 
-   char buffer[33000];	
+   char buffer[PATH_MAX];	
    const char suffix[] = ".so";
 
    strcpy( buffer, PKGLIBDIR PKG_VERSION "/cpi" );
@@ -324,8 +393,8 @@ static unicap_status_t enumerate_devices( int *count )
       dlhandle = dlopen( filename, RTLD_NOW );
       if( !dlhandle )
       {
-	 TRACE( "cpi load: %s\n", dlerror() );
-	 perror( "unicap CPI load" );
+/* 	 TRACE( "cpi load: %s\n", dlerror() ); */
+	 fprintf( stderr, "cpi load '%s': %s\n", filename, dlerror() );
 	 continue;
       }
 		
@@ -447,7 +516,8 @@ static unicap_status_t open_cpi( unicap_handle_t *unicap_handle, unicap_device_t
       handle->cb_info[ i ].user_ptr = NULL;
    }
 
-   sprintf( handle->device_uid, "%s", device->identifier );
+   handle->lock = malloc( sizeof( struct unicap_device_lock ) );
+   memset( handle->lock, 0x0, sizeof( struct unicap_device_lock ) );
 
    dlhandle = dlopen( device->cpi_layer, RTLD_LAZY );
    if( !dlhandle )
@@ -582,7 +652,8 @@ static unicap_status_t open_cpi( unicap_handle_t *unicap_handle, unicap_device_t
       handle->cb_info[ i ].user_ptr = NULL;
    }
 
-   sprintf( handle->device_uid, "%s", device->identifier );
+   handle->lock = malloc( sizeof( struct unicap_device_lock ) );
+   memset( handle->lock, 0x0, sizeof( struct unicap_device_lock ) );
 
    sscanf( device->cpi_layer, "%d", &idx_cpi );
    if( ( idx_cpi < 0 ) || ( idx_cpi > UNICAP_STATIC_CPI_COUNT ) )
@@ -618,8 +689,19 @@ static unicap_status_t open_cpi( unicap_handle_t *unicap_handle, unicap_device_t
 
 unicap_status_t unicap_open( unicap_handle_t *unicap_handle, unicap_device_t *device )
 {
-   unicap_status_t status = open_cpi( unicap_handle, device );
-   
+   unicap_status_t status;
+   struct unicap_device_cache *cache_entry;
+
+   unicap_initialize();
+
+   cache_entry = lookup_device_cache( device );
+   if( !cache_entry || !cache_entry->handle ){
+      status = open_cpi( unicap_handle, device );
+      store_device_cache( device, *unicap_handle );
+   } else {
+      *unicap_handle = unicap_clone_handle( cache_entry->handle );
+      TRACE( "device in cache refcount = %d\n", *cache_entry->handle->ref_count );
+   }   
 
    return status;
 }
@@ -628,7 +710,7 @@ void unicap_cpi_register_event_notification( void* _func,
 					     void *data, unicap_handle_t handle )
 {
    cpi_set_event_notify_t func = (cpi_set_event_notify_t)_func;
-   
+
    func( data, unicap_event_callback, unicap_clone_handle( handle ) );
 }
 
@@ -656,9 +738,10 @@ unicap_handle_t unicap_clone_handle( unicap_handle_t handle )
       return NULL;
    }
 	
+   _unicap_acquire_cpi_lock( handle->sem_id );
    memcpy( cloned_handle, handle, sizeof( struct _unicap_handle ) );
-	
    *(handle->ref_count) = *(handle->ref_count) + 1;
+   _unicap_release_cpi_lock( handle->sem_id );
    TRACE( "clone handle = %p: handle = %p refcount = %d\n", cloned_handle, handle, *(handle->ref_count) );
 
    return cloned_handle;
@@ -704,6 +787,13 @@ unicap_status_t unicap_close( unicap_handle_t handle )
    *(handle->ref_count) = *(handle->ref_count) - 1;
    TRACE( "close: handle = %p new refcount = %d\n", handle, *(handle->ref_count ) );
 
+   if( *(handle->ref_count) == 1 )
+   {
+      free_device_cache_entries();
+      free( handle );
+      return STATUS_SUCCESS;
+   }   
+
    if( *(handle->ref_count) == 0 )
    {
       unicap_unlock_properties( handle );
@@ -719,6 +809,8 @@ unicap_status_t unicap_close( unicap_handle_t handle )
       }
       _unicap_release_cpi_lock( handle->sem_id );
       free( handle->ref_count );
+      free( handle->cb_info );
+      free( handle->lock );
    }
    free( handle );
    return STATUS_SUCCESS;
@@ -1115,12 +1207,12 @@ unicap_status_t unicap_start_capture( unicap_handle_t handle )
       return STATUS_INVALID_PARAMETER;
    }
 
-   if( !handle->have_stream_lock )
+   if( !handle->lock->have_stream_lock )
    {
       status = unicap_lock_stream( handle );
       if( SUCCESS( status ) )
       {
-	 handle->temporary_stream_lock = 1;
+	 handle->lock->temporary_stream_lock = 1;
       }
    }
 
@@ -1148,15 +1240,15 @@ unicap_status_t unicap_stop_capture( unicap_handle_t handle )
       return STATUS_INVALID_PARAMETER;
    }
 
-   if( !handle->have_stream_lock )
+   if( !handle->lock->have_stream_lock )
    {
       status = STATUS_PERMISSION_DENIED;
    }
    else
    {
-      if( handle->temporary_stream_lock )
+      if( handle->lock->temporary_stream_lock )
       {
-	 handle->temporary_stream_lock = 0;
+	 handle->lock->temporary_stream_lock = 0;
 	 unicap_unlock_stream( handle );
       }
 		
@@ -1179,7 +1271,7 @@ unicap_status_t unicap_queue_buffer( unicap_handle_t handle, unicap_data_buffer_
    }
 	
 
-   if( !handle->have_stream_lock )
+   if( !handle->lock->have_stream_lock )
    {
       status = STATUS_PERMISSION_DENIED;
    }
@@ -1202,7 +1294,7 @@ unicap_status_t unicap_dequeue_buffer( unicap_handle_t handle, unicap_data_buffe
       return STATUS_INVALID_PARAMETER;
    }
 
-   if( !handle->have_stream_lock )
+   if( !handle->lock->have_stream_lock )
    {
       status = STATUS_PERMISSION_DENIED;
    }
@@ -1226,7 +1318,7 @@ unicap_status_t unicap_wait_buffer( unicap_handle_t handle, unicap_data_buffer_t
       return STATUS_INVALID_PARAMETER;
    }
 
-   if( !handle->have_stream_lock )
+   if( !handle->lock->have_stream_lock )
    {
       status = STATUS_PERMISSION_DENIED;
    }
@@ -1277,6 +1369,8 @@ int unicap_is_stream_locked( unicap_device_t *device )
    int val;
    char uid[2048];
 	
+   unicap_initialize();
+
 #ifndef UNICAP_THREAD_LOCKING
    return 0;
 #endif
@@ -1291,11 +1385,21 @@ int unicap_is_stream_locked( unicap_device_t *device )
       return 0;
    }
 	
-   val = semctl( sem_id, 0, GETVAL );
+   val = semctl( sem_id, 0, GETVAL ) ? 0 : UNICAP_FLAGS_LOCKED;
+   if( val == UNICAP_FLAGS_LOCKED ){
+      struct unicap_device_cache *cache_entry;
+      
+      cache_entry = lookup_device_cache( device );
+      if( cache_entry && cache_entry->handle ){
+	 if( cache_entry->handle->lock->have_stream_lock ){
+	    val |= UNICAP_FLAGS_LOCK_CURRENT_PROCESS;
+	 }
+      }
+   } 
 
    TRACE( "STREAM IS LOCKED %s%s >>>> %d (%d) key + %d\n", device->identifier, device->cpi_layer, val == 0, val, key );
 
-   return val == 0;
+   return val;
 }
 
 
@@ -1307,15 +1411,14 @@ unicap_status_t unicap_lock_stream( unicap_handle_t handle )
    char uid[2048];
    
 #ifndef UNICAP_THREAD_LOCKING
-   handle->have_stream_lock = 1;
+   handle->lock->have_stream_lock = 1;
    return STATUS_SUCCESS;
 #endif
    
    sprintf( uid, "%s", handle->device.identifier );
    key = uidtok( uid );
-/*    key = uidtok( handle->device_uid ); */
 
-   if( !handle->have_stream_lock )
+   if( !handle->lock->have_stream_lock )
    {
       struct semid_ds semds;
       union semun_linux semun;
@@ -1348,8 +1451,8 @@ unicap_status_t unicap_lock_stream( unicap_handle_t handle )
 	 }
 	 else
 	 {
-	    handle->stream_sem_id = sem_id;
-	    handle->have_stream_lock = 1;
+	    handle->lock->stream_sem_id = sem_id;
+	    handle->lock->have_stream_lock = 1;
 	 }
 	 _unicap_release_cpi_lock( handle->sem_id );
       }
@@ -1370,14 +1473,14 @@ unicap_status_t unicap_lock_stream( unicap_handle_t handle )
 	 }
 	 else
 	 {
-	    handle->stream_sem_id = sem_id;
-	    handle->have_stream_lock = 1;
+	    handle->lock->stream_sem_id = sem_id;
+	    handle->lock->have_stream_lock = 1;
 	 }
 	 _unicap_release_cpi_lock( handle->sem_id );
       }
    }
 
-   TRACE( "LOCK STREAM >>>> %d key = %d\n", handle->have_stream_lock, key );
+   TRACE( "LOCK STREAM >>>> %d key = %d\n", handle->lock->have_stream_lock, key );
 	
    return status;
 }
@@ -1387,24 +1490,24 @@ unicap_status_t unicap_unlock_stream( unicap_handle_t handle )
    unicap_status_t status = STATUS_SUCCESS;
 
 #ifndef UNICAP_THREAD_LOCKING
-   handle->have_stream_lock = 0;
+   handle->lock->have_stream_lock = 0;
    return STATUS_SUCCESS;
 #endif
 	
-   if( handle->have_stream_lock )
+   if( handle->lock->have_stream_lock )
    {
       struct sembuf sops;
       sops.sem_num = 0;
       sops.sem_op = 1;
       sops.sem_flg = SEM_UNDO;
 		
-      if( semop( handle->stream_sem_id, &sops, 1 ) < 0 )
+      if( semop( handle->lock->stream_sem_id, &sops, 1 ) < 0 )
       {
 	 status = STATUS_FAILURE;
       }
       else
       {
-	 handle->have_stream_lock = 0;
+	 handle->lock->have_stream_lock = 0;
       }
 		
    }
@@ -1413,7 +1516,7 @@ unicap_status_t unicap_unlock_stream( unicap_handle_t handle )
       status = STATUS_PERMISSION_DENIED;
    }
 	
-   TRACE( "UNLOCK STREAM >>>> %d\n", handle->have_stream_lock );
+   TRACE( "UNLOCK STREAM >>>> %d\n", handle->lock->have_stream_lock );
 
    return status;
 }
@@ -1438,6 +1541,8 @@ unicap_status_t unicap_register_callback( unicap_handle_t handle,
 {
    unicap_status_t status = STATUS_FAILURE;
 
+   TRACE( "register callback: %p\n", func );
+
    if( ( event < UNICAP_EVENT_FIRST ) || 
        ( event >= UNICAP_EVENT_LAST ) )
    {
@@ -1446,7 +1551,7 @@ unicap_status_t unicap_register_callback( unicap_handle_t handle,
 	
    _unicap_acquire_cpi_lock( handle->sem_id );
 
-   if( handle->cb_info[event].func != NULL )
+   if( !handle->cb_info || ( handle->cb_info[event].func != NULL ) )
    {
       status = STATUS_FAILURE;
    }
@@ -1467,6 +1572,7 @@ unicap_status_t unicap_unregister_callback( unicap_handle_t handle,
 {
    unicap_status_t status = STATUS_FAILURE;
 
+
    if( ( event < UNICAP_EVENT_FIRST ) || 
        ( event >= UNICAP_EVENT_LAST ) )
    {
@@ -1491,3 +1597,7 @@ unicap_status_t unicap_unregister_callback( unicap_handle_t handle,
    return status;
 }
 	
+int unicap_get_ref_count( unicap_handle_t handle )
+{
+   return *handle->ref_count;
+}
