@@ -338,7 +338,7 @@ static void property_changed_cb( GtkWidget *widget, unicap_property_t *property,
    for( i = 0; i < property->relations_count; i++ )
    {
       GList *entry;
-      
+
       entry = g_list_find_custom( ugtk->property_list, property->relations[i], (GCompareFunc)strcmp );
       if( entry )
       {
@@ -488,7 +488,8 @@ static gboolean save_device_defaults( UnicapgtkPropertyDialog *ugtk, gboolean ov
       f = fopen( ini_path, "w" );
       if( f )
       {
-	 fwrite( data, size, 1, f );
+	 int ignore;
+	 ignore = fwrite( data, size, 1, f );
 	 fclose( f );
 
 	 result = TRUE;
