@@ -123,6 +123,7 @@ struct _UnicapgtkVideoDisplay
       backend_unlock_t backend_unlock;
       backend_display_image_t backend_display_image;
       backend_size_allocate_t backend_size_allocate;
+      backend_set_crop_t backend_set_crop;
       backend_set_scale_to_fit_t backend_set_scale_to_fit;
       backend_set_pause_state_t backend_set_pause_state;
       backend_get_flags_t backend_get_flags;
@@ -649,7 +650,7 @@ static gint timeout_cb( UnicapgtkVideoDisplay *ugtk )
 			unicapgtk_video_display_signals[ UNICAPGTK_VIDEO_DISPLAY_PREDISPLAY_SIGNAL],
 			0,
 			(gpointer)&data_buffer );
-	 
+
 	 ugtk->backend_display_image( ugtk->backend_data );
 /* 	 } */
 /* 	 else */
@@ -679,9 +680,19 @@ static void new_frame_cb( unicap_event_t event, unicap_handle_t handle, unicap_d
 
       if( ugtk->backend )
       {
+/* 	 printf( "-->%p %p %p\n", ugtk->backend_lock, ugtk->backend_update_image, ugtk->backend_unlock ); */
 	 ugtk->backend_lock( ugtk->backend_data );
+/* 	 printf( "--%p %p %p\n", ugtk->backend_lock, ugtk->backend_update_image, ugtk->backend_unlock ); */
+	 if( !ugtk->backend_update_image ){
+	    TRACE( "!!!!\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n" );
+	    ugtk->backend_unlock( ugtk->backend_data );
+	    return;
+	 }
+	    
 	 ugtk->backend_update_image( ugtk->backend_data, buffer, NULL );
+/* 	 printf( "--%p %p %p\n", ugtk->backend_lock, ugtk->backend_update_image, ugtk->backend_unlock ); */
 	 ugtk->backend_unlock( ugtk->backend_data );
+/*  	 printf( "<--\n" ); */
       }
    }
 }
@@ -749,7 +760,11 @@ gboolean unicapgtk_video_display_start( UnicapgtkVideoDisplay *ugtk )
    ugtk->disp_time.tv_sec = 0;
    ugtk->disp_time.tv_usec = 0;
 
-   unicap_register_callback( ugtk->unicap_handle, UNICAP_EVENT_NEW_FRAME, (unicap_callback_t)new_frame_cb, ugtk );
+   unicap_unregister_callback( ugtk->unicap_handle, UNICAP_EVENT_NEW_FRAME );
+   if( !SUCCESS( unicap_register_callback( ugtk->unicap_handle, UNICAP_EVENT_NEW_FRAME, (unicap_callback_t)new_frame_cb, ugtk ) ) ){
+      TRACE( "Failed to register callback\n" );
+      return FALSE;
+   }
 
    ugtk->display_timeout_tag = g_timeout_add( 30, (GtkFunction)timeout_cb, ugtk );
 
@@ -776,7 +791,6 @@ void unicapgtk_video_display_stop( UnicapgtkVideoDisplay *ugtk )
    int cbuf;
    if( !ugtk->capture_running )
    {
-/*       g_warning( "video_display not running\n" ); */
       return;
    }
 
@@ -789,7 +803,6 @@ void unicapgtk_video_display_stop( UnicapgtkVideoDisplay *ugtk )
    }
 	
    unicap_stop_capture( ugtk->unicap_handle );
-   
 
    if( ugtk->still_image_buffer.data )   //
    {
@@ -1053,6 +1066,10 @@ void unicapgtk_video_display_set_crop( UnicapgtkVideoDisplay *ugtk,
    ugtk->crop_width = crop->crop_width;
    ugtk->crop_height = crop->crop_height;
 
+   if( ugtk->backend ){
+      ugtk->backend_set_crop( ugtk->backend_data, ugtk->crop_x, ugtk->crop_y, ugtk->crop_width, ugtk->crop_height );
+   }
+
    if (! ugtk->scale_to_fit)
    {
       gtk_widget_set_size_request( ugtk->drawing_area, 
@@ -1442,12 +1459,17 @@ static gboolean da_realize_cb( GtkWidget *da, UnicapgtkVideoDisplay *ugtk )
 
 static void set_backend( UnicapgtkVideoDisplay *ugtk )
 {
+   int old_running = ugtk->capture_running;
+/*    printf( "sb\n" ); */
+   if( ugtk->capture_running ){
+      unicap_unregister_callback( ugtk->unicap_handle, UNICAP_EVENT_NEW_FRAME );
+   }
    if( ugtk->backend )
    {
       ugtk->backend_destroy( ugtk->backend_data );
       ugtk->backend = NULL;
-   }      
-
+   }
+/*    printf( "backend = NULL\n" ); */
    ugtk->backend_destroy = NULL;
    ugtk->backend_update_image = NULL;
    ugtk->backend_redraw = NULL;
@@ -1456,6 +1478,10 @@ static void set_backend( UnicapgtkVideoDisplay *ugtk )
 
    if( ! GTK_WIDGET_REALIZED( ugtk ))
    {
+      if( old_running ){
+	 unicap_register_callback( ugtk->unicap_handle, UNICAP_EVENT_NEW_FRAME, (unicap_callback_t)new_frame_cb, ugtk );
+      }
+/*       printf( "--setbackend (!realized)\n" ); */
       return;
    }
 
@@ -1464,6 +1490,10 @@ static void set_backend( UnicapgtkVideoDisplay *ugtk )
    {
       if( ( ugtk->requested_backend == BACKEND_XV ) || !backend_gtk_init( ugtk->drawing_area, &ugtk->format, &ugtk->backend_data, NULL ) )
       {
+	 if( old_running ){
+	    unicap_register_callback( ugtk->unicap_handle, UNICAP_EVENT_NEW_FRAME, (unicap_callback_t)new_frame_cb, ugtk );
+	 }
+/* 	 printf( "--setbackend (!initialized)\n" );	  */
 	 return;
       }
       else
@@ -1472,6 +1502,7 @@ static void set_backend( UnicapgtkVideoDisplay *ugtk )
 	 ugtk->backend_update_image = backend_gtk_update_image;
 	 ugtk->backend_redraw = backend_gtk_redraw;
 	 ugtk->backend_size_allocate = backend_gtk_size_allocate;
+	 ugtk->backend_set_crop = backend_gtk_set_crop;
 	 ugtk->backend_expose = backend_gtk_expose_event;
 	 ugtk->backend_get_image_data = backend_gtk_get_image_data;
 	 ugtk->backend_lock = backend_gtk_lock;
@@ -1491,6 +1522,7 @@ static void set_backend( UnicapgtkVideoDisplay *ugtk )
       ugtk->backend_update_image = backend_xv_update_image;
       ugtk->backend_redraw = backend_xv_redraw;
       ugtk->backend_size_allocate = backend_xv_size_allocate;
+      ugtk->backend_set_crop = backend_xv_set_crop;
       ugtk->backend_expose = backend_xv_expose_event;
       ugtk->backend_get_image_data = backend_xv_get_image_data;
       ugtk->backend_lock = backend_xv_lock;
@@ -1508,6 +1540,11 @@ static void set_backend( UnicapgtkVideoDisplay *ugtk )
 
    g_object_notify( G_OBJECT( ugtk ), "backend" );
 
+   if( old_running ){
+      unicap_register_callback( ugtk->unicap_handle, UNICAP_EVENT_NEW_FRAME, (unicap_callback_t)new_frame_cb, ugtk );
+   }
+
+/*    printf( "--setbackend\n" ); */
 }
 
 /**
